import { DataValue, RequestParams, ResponseResult, ContextExpression, FieldDescriptor as UnsureTypeField, FilterDescriptor as UnsureTypeFilter, ViewFieldDescriptor as UnsureTypeViewField } from '../vendors/organik';
interface UnknownField extends UnsureTypeField {
    dynamic?: boolean;
}
interface NumberField extends UnknownField {
    dataType: 'integer' | 'float';
    max?: number;
    min?: number;
}
interface StringField extends UnknownField {
    dataType: 'string' | 'text';
    max?: number;
    min?: number;
    pattern?: RegExp;
}
interface EnumFieldOption {
    name: string;
    label: string;
    value: number | string;
    hint?: string;
    available?: ContextExpression;
    color?: string;
    icon?: string;
}
declare type EnumFieldOptionGetter = () => Promise<ResponseResult<EnumFieldOption[]>>;
interface EnumField extends UnknownField {
    dataType: 'enum';
    options: EnumFieldOption[] | EnumFieldOptionGetter;
}
interface MultiEnumField extends UnknownField {
    dataType: 'multi-enum';
    options: EnumFieldOption[] | EnumFieldOptionGetter;
}
declare type RelationFieldType = 'o2o' | 'o2m' | 'm2m' | 'm2o';
interface StaticRelationField extends UnknownField {
    dataType: RelationFieldType;
    dynamic: false;
}
declare type RequestFunction = (params: RequestParams) => Promise<ResponseResult>;
interface DynamicRelationField extends UnknownField {
    dataType: RelationFieldType;
    dynamic: true;
    referenceValueGetter: RequestFunction;
    relatedListGetter: RequestFunction;
    relatedPrimaryKey: string;
    relatedLabelKey?: string;
    relatedValueKey?: string;
}
declare type RelationField = StaticRelationField | DynamicRelationField;
declare type FieldDescriptor = UnknownField | NumberField | StringField | EnumField | MultiEnumField | RelationField;
interface UnknownFilter extends UnsureTypeFilter {
    placeholder?: string;
    dynamic?: boolean;
    hint?: string;
}
declare type Filterize<T extends UnknownField = UnknownField> = Omit<T, 'readonly'>;
declare type NumberFilter = UnknownFilter & Filterize<NumberField>;
declare type StringFilter = UnknownFilter & Filterize<StringField>;
declare type EnumFilter = UnknownFilter & Filterize<EnumField>;
declare type MultiEnumFilter = UnknownFilter & Filterize<MultiEnumField>;
declare type StaticRelationFilter = UnknownFilter & Filterize<StaticRelationField>;
declare type DynamicRelationFilter = UnknownFilter & Filterize<DynamicRelationField>;
declare type RelationFilter = StaticRelationFilter | DynamicRelationFilter;
declare type FilterDescriptor = UnknownFilter | NumberFilter | StringFilter | EnumFilter | MultiEnumFilter | RelationFilter;
interface UnknownViewField extends UnsureTypeViewField {
    placeholder?: string;
    dynamic?: boolean;
    hint?: string;
    available?: ContextExpression;
    formatter?: (value: DataValue) => DataValue;
}
declare type NumberViewField = UnknownViewField & NumberField;
declare type StringViewField = UnknownViewField & StringField;
declare type EnumViewField = UnknownViewField & EnumField;
declare type MultiEnumViewField = UnknownViewField & MultiEnumField;
declare type StaticRelationViewField = UnknownViewField & StaticRelationField;
declare type DynamicRelationViewFIeld = UnknownViewField & DynamicRelationField;
declare type RelationViewField = StaticRelationViewField | DynamicRelationViewFIeld;
declare type ViewFieldDescriptor = UnknownViewField | NumberViewField | StringViewField | EnumViewField | MultiEnumViewField | RelationViewField;
export { NumberField, StringField, EnumFieldOption, EnumFieldOptionGetter, EnumField, MultiEnumField, DynamicRelationField, FieldDescriptor, UnknownFilter, DynamicRelationFilter, FilterDescriptor, UnknownViewField, RelationViewField, ViewFieldDescriptor, };
