import { isPlainObject, clone } from '@ntks/toolbox';
import { resolveInput } from '../core/input';
import { ValueContext } from './value';
class SearchContext extends ValueContext {
    constructor({ filters, defaultValue }) {
        super({ defaultValue });
        this.condition = defaultValue;
        const resolvedFilters = filters.map(filter => resolveInput(filter));
        this.filters = resolvedFilters;
        this.filterMap = resolvedFilters.reduce((prev, filter) => (Object.assign(Object.assign({}, prev), { [filter.name]: filter })), {});
        super.on('change', value => (this.condition = value));
    }
    setValue(value) {
        if (!isPlainObject(value)) {
            return;
        }
        super.setValue(value);
    }
    getFilters() {
        return clone(this.filters);
    }
    getFilterValue(name) {
        return this.condition[name];
    }
    setFilterValue(name, value) {
        if (this.filterMap[name] === undefined) {
            return;
        }
        super.setValue(Object.assign(Object.assign({}, this.condition), { [name]: value }));
        this.emit('filterChange', { name, value });
    }
}
export { SearchContext };
