import { isString, generateRandomId, clone, pick } from '@ntks/toolbox';
import { resolveAction, setViewContext, resolveFields, runExpression, } from '../../core';
import { resolveInput } from '../../core/input';
import { ValueContext } from '../value';
class ViewContext extends ValueContext {
    constructor(moduleContext, options) {
        super(pick(options, ['defaultValue', 'initialValue']));
        this.busy = false;
        this.id = generateRandomId('OrganikViewContext');
        this.moduleContext = moduleContext;
        this.options = options;
        this.fields = resolveFields((options.fields || []).map(field => resolveInput(field)), moduleContext.getModel());
        this.opener = options.opener;
        const actions = (options.actions || [])
            .map(resolveAction)
            .filter(action => !!action);
        const actionContextGroups = {};
        actions.forEach(action => {
            const contextType = action.context || 'single';
            if (!actionContextGroups[contextType]) {
                actionContextGroups[contextType] = [];
            }
            actionContextGroups[contextType].push(action);
        });
        this.actions = actions;
        this.actionContextGroups = actionContextGroups;
        this.dataSource = clone(options.initialValue || options.defaultValue);
        setViewContext(this);
    }
    getId() {
        return this.id;
    }
    getModuleContext() {
        return this.moduleContext;
    }
    /**
     * Get opener opened this view (pop-up view)
     *
     * @returns the opener that open this view
     */
    getOpener() {
        return this.opener;
    }
    getComponents() {
        return this.moduleContext.getComponents();
    }
    getView() {
        return this.options;
    }
    getFields() {
        return clone(this.fields);
    }
    getActions() {
        return this.actions.filter(action => isString(action.available)
            ? !!runExpression({ dataSource: this.getDataSource(), value: this.getValue() }, action.available, false)
            : action);
    }
    getActionsByContextType(contextType) {
        return this.actionContextGroups[contextType] || [];
    }
    getActionsAuthority() {
        return this.options.actionsAuthority;
    }
    getConfig() {
        return (this.options.config || {});
    }
    getDataSource() {
        return this.dataSource;
    }
    setDataSource(data) {
        this.dataSource = data;
    }
    getBusy() {
        return this.busy;
    }
    setBusy(busy) {
        this.busy = busy;
        this.emit('busyChange', busy);
    }
}
export { ViewContext };
