import { pick, clone } from '@ntks/toolbox';
import { createInputValidator, } from '../../core';
import { ViewContext } from './base';
class ObjectViewContext extends ViewContext {
    constructor(moduleContext, options) {
        super(moduleContext, Object.assign(Object.assign({}, options), { defaultValue: {}, initialValue: options.initialValue || {} }));
        this.invalidFieldMap = {};
        this.modified = false;
        this.parent = options.parent;
        this.indexInParent = options.indexInParent === undefined ? -1 : options.indexInParent;
        this.validationTiming = options.validate || 'immediate';
        this.shorthandNames = pick(options, ['getOne', 'insert', 'update']);
        this.validatorMap = this.getFields().reduce((prev, field) => (Object.assign(Object.assign({}, prev), { [field.name]: createInputValidator(field) })), {});
    }
    validateFieldValue(name, value) {
        const validator = this.validatorMap[name];
        if (validator === undefined) {
            return;
        }
        const validationResult = validator.validate(value, {
            dataSource: this.getDataSource(),
            value: this.getValue(),
        });
        if (validationResult.success) {
            delete this.invalidFieldMap[name];
        }
        else {
            this.invalidFieldMap[name] = true;
        }
        this.emit('fieldValidate', { name, result: validationResult });
    }
    getParent() {
        return this.parent;
    }
    getIndexInParent() {
        return this.indexInParent;
    }
    setDataSource(data) {
        this.modified = false;
        const copy = clone(data);
        super.setDataSource(copy);
        this.emit('dataChange', data);
    }
    setValue(value) {
        this.modified = true;
        super.setValue(value);
    }
    getFieldValue(name) {
        return this.getValue()[name];
    }
    setFieldValue(name, value) {
        if (this.validatorMap[name] === undefined) {
            return;
        }
        if (this.validationTiming === 'immediate') {
            this.validateFieldValue(name, value);
        }
        this.setValue(Object.assign(Object.assign({}, this.getValue()), { [name]: value }));
        this.emit('fieldChange', { name, value });
    }
    setFieldChecker(name, checker) {
        if (!this.validatorMap[name]) {
            return;
        }
        this.validatorMap[name].addChecker(checker);
    }
    isModified() {
        return this.modified;
    }
    submit() {
        if (this.validationTiming === 'submit') {
            this.getFields().forEach(({ name }) => this.validateFieldValue(name, this.getFieldValue(name)));
        }
        if (Object.keys(this.invalidFieldMap).length > 0) {
            return;
        }
        this.modified = false;
        super.submit();
    }
    reset() {
        super.reset();
        this.invalidFieldMap = {};
        this.modified = false;
    }
    getOne(params, success, fail) {
        return this.getModuleContext().execute(this.shorthandNames.getOne || 'getOne', params, success, fail);
    }
    insert(params, success, fail) {
        return this.getModuleContext().execute(this.shorthandNames.insert || 'insert', params, success, fail);
    }
    update(params, success, fail) {
        return this.getModuleContext().execute(this.shorthandNames.update || 'update', params, success, fail);
    }
}
export { ObjectViewContext };
