import { ComponentCtor } from '../component';
import { ModelDescriptor } from './model';
import { ServerAction } from './action';
import { ViewComponentRenderer } from './view';
declare type ModuleResourceType = 'services' | 'utils' | 'widgets';
declare type ModuleResources = Partial<Record<ModuleResourceType, any>>;
declare type ModuleDependencies = Record<string, ModuleResources>;
declare type ModuleComponentRefs = Record<string, boolean | string>;
declare type AsyncFunction = (...args: any[]) => Promise<any>;
declare type UnsureModuleActions<K extends string = string> = Record<K, ServerAction | AsyncFunction>;
declare type ModuleActions<K extends string = string> = Record<K, ServerAction>;
declare type ModuleViews = Record<string, ViewComponentRenderer>;
declare type ModuleDescriptor<K extends string = string> = {
    name: string;
    model?: ModelDescriptor;
    actions?: UnsureModuleActions<K>;
    views?: ModuleViews;
    imports?: string[];
    exports?: Partial<Record<ModuleResourceType, Record<string, any>>>;
    components?: ModuleComponentRefs;
};
declare type ModuleComponents = Record<string, ComponentCtor>;
declare type ResolvedModule = Required<Omit<ModuleDescriptor, 'name' | 'actions' | 'components'>> & {
    actions: ModuleActions;
    dependencies: Record<string, any>;
    componentRefs: ModuleComponentRefs;
    components: ModuleComponents;
};
export { AsyncFunction, ModuleResourceType, ModuleResources, ModuleDependencies, UnsureModuleActions, ModuleActions, ModuleViews, ModuleDescriptor, ModuleComponents, ResolvedModule, };
